/*
 * File:      robotSoldierPeripheral.ino
 * Description: BLE peripheral device script for a controllable robot soldier.
 *            Use BLEControllerCentral.ino for a controller divice.
 *            A: raise the right arm
 *            B: lower the right arm
 *            C: raise the left arm
 *            D: lower the left arm
 *            E/F: -
 *            K: make a beep and blink the LED
 *            joyX: turn the head
 *            joyY: walk forward/backward
 * Date:      9/24/2023 (modified 11/21/2023)
 * Author:    Sayaka
 * Hardware  MCU: ESP32 (ESP32 DEVKIT V4 Board)
 *            RC servo x3
 *            DC motor x1
 *            LED x2
 *            passive buzzzer x1
 */
#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLE2902.h>

#include <ESP32Servo.h>

#define SERVER_NAME "ROBOT_SOLDIER"
#define SPI_SPEED 115200         // SPI speed

/* Notes */
#define NOTE_B0 31
#define NOTE_C1 33
#define NOTE_CS1 35
#define NOTE_D1 37
#define NOTE_DS1 39
#define NOTE_E1 41
#define NOTE_F1 44
#define NOTE_FS1 46
#define NOTE_G1 49
#define NOTE_GS1 52
#define NOTE_A1 55
#define NOTE_AS1 58
#define NOTE_B1 62
#define NOTE_C2 65
#define NOTE_CS2 69
#define NOTE_D2 73
#define NOTE_DS2 78
#define NOTE_E2 82
#define NOTE_F2 87
#define NOTE_FS2 93
#define NOTE_G2 98
#define NOTE_GS2 104
#define NOTE_A2 110
#define NOTE_AS2 117
#define NOTE_B2 123
#define NOTE_C3 131
#define NOTE_CS3 139
#define NOTE_D3 147
#define NOTE_DS3 156
#define NOTE_E3 165
#define NOTE_F3 175
#define NOTE_FS3 185
#define NOTE_G3 196
#define NOTE_GS3 208
#define NOTE_A3 220
#define NOTE_AS3 233
#define NOTE_B3 247
#define NOTE_C4 262
#define NOTE_CS4 277
#define NOTE_D4 294
#define NOTE_DS4 311
#define NOTE_E4 330
#define NOTE_F4 349
#define NOTE_FS4 370
#define NOTE_G4 392
#define NOTE_GS4 415
#define NOTE_A4 440
#define NOTE_AS 455
#define NOTE_AS4 466
#define NOTE_B4 494
#define NOTE_C5 523
#define NOTE_CS5 554
#define NOTE_D5 587
#define NOTE_DS5 622
#define NOTE_E5 659
#define NOTE_F5 698
#define NOTE_FS5 740
#define NOTE_G5 784
#define NOTE_GSH 830
#define NOTE_GS5 831
#define NOTE_A5 880
#define NOTE_AS5 932
#define NOTE_B5 988
#define NOTE_C6 1047
#define NOTE_CS6 1109
#define NOTE_D6 1175
#define NOTE_DS6 1245
#define NOTE_E6 1319
#define NOTE_F6 1397
#define NOTE_FS6 1480
#define NOTE_G6 1568
#define NOTE_GS6 1661
#define NOTE_A6 1760
#define NOTE_AS6 1865
#define NOTE_B6 1976
#define NOTE_C7 2093
#define NOTE_CS7 2217
#define NOTE_D7 2349
#define NOTE_DS7 2489
#define NOTE_E7 2637
#define NOTE_F7 2794
#define NOTE_FS7 2960
#define NOTE_G7 3136
#define NOTE_GS7 3322
#define NOTE_A7 3520
#define NOTE_AS7 3729
#define NOTE_B7 3951
#define NOTE_C8 4186
#define NOTE_CS8 4435
#define NOTE_D8 4699
#define NOTE_DS8 4978
#define NOTE_E8 5274
#define NOTE_F8 5587

/* BLE */
// See the following for generating UUIDs:
// https://www.uuidgenerator.net/
#define SERVICE_UUID "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID_RX "beb5483e-36e1-4688-b7f5-ea07361b26a8"

BLECharacteristic *pCharacteristicRX;
bool deviceConnected = false;
bool oldDeviceConnected = false;

/* Mode */
bool chatting = false;
bool singing = false;
bool helloing = false;
bool automode1 = false;
bool automode2 = false;

/* Auto mode */
unsigned long autoStartTime;
unsigned long interval1 = 120000;
unsigned long interval2 = 600000;
unsigned long lastAutoTime1;
unsigned long lastAutoTime2;
bool autoState = false;
int doHelloStep = 0;
int oddCheck = 0; // for control servo motor speed
unsigned long lastHelloTime;

/* LED */
const int ledPin[] = { 19, 18 };  // upper LED, lower LED

unsigned long lastLedTime[] = { 0, 0 };
int lastLedState[] = { 0, 0 };
int ledStep[] = { 0, 0 };
bool ledStepEnd[] = { false, false };

/* buzzer  */
const int bzPin = 14;
const int bzCh = 5;  // Ch0/1 interfere servo

unsigned long lastBzTime = 0;
bool bzState = false;
bool lastBzState = false;
int bzStep = 0;
bool bzStepEnd = false;

/* servo motor  */
const int headPin = 32;  // servo motor for head
const int armRPin = 33;  // servo motor for right arm
const int armLPin = 25;  // servo motor forleft arm

Servo svHead;
Servo svArmR;
Servo svArmL;

const int headMaxDeg = 170;
const int headMinDeg = 10;
const int armRMaxDeg = 130;
const int armRMinDeg = 40;
const int armLMaxDeg = 130;
const int armLMinDeg = 40;

int headDeg = 90;
int armRDeg = armRMinDeg;
int armLDeg = armLMaxDeg;
bool armRup = false;
bool armRdown = false;
bool armLup = false;
bool armLdown = false;

/* DC motor  */
const int leg1Pin = 26;  //
const int leg2Pin = 27;  //
int speed = 0;
int joyThreshold = 50;

/* recieve data */
struct tmpData {
  uint8_t abcdBtn;
  uint8_t efkBtn;
  uint16_t xyJoy[2];
};
struct tmpData data;

const char buttonName[7] = { 'A', 'B', 'C', 'D', 'E', 'F', 'K' };
uint16_t joyOffset[2];
bool kBtn = false;

/* Declaration of Functions */
void doInitVar();


/******************** < Callback classes > ********************/
class ServerCallbacks : public BLEServerCallbacks {
  void onConnect(BLEServer *pServer) {
    deviceConnected = true;
  }
  void onDisconnect(BLEServer *pServer) {
    deviceConnected = false;
  }
};


class ReceiveCallbacks : public BLECharacteristicCallbacks {
  void onWrite(BLECharacteristic *pCharacteristicRX) {

    std::string rxValue = pCharacteristicRX->getValue();
    // if (rxValue.length() > 0) {
    //     Serial.print("RECEIVED DATA: ");
    //     for (int i = 0; i < rxValue.length(); i++) {
    //         Serial.print(rxValue[i], BIN);
    //         Serial.print(",");
    //     }
    //     Serial.println();
    // }

    memcpy(&data, &rxValue[0], sizeof(tmpData));
    if (data.abcdBtn == 0xFF && data.efkBtn == 0xFF) {  // When the offset value cames.
      joyOffset[0] = data.xyJoy[0];
      joyOffset[1] = data.xyJoy[1];
      chatting = true;  // sign that offset value has been obtained
    } else {

      if (!automode1 && !automode2) { // When not in auto mode.
        // moving head
        if (data.xyJoy[0] > joyOffset[0] + joyThreshold) {  // move right　0~90deg
          headDeg = map(data.xyJoy[0], joyOffset[0], 0xFFF, 90, 0);
        } else if (data.xyJoy[0] < joyOffset[0] - joyThreshold) {  //　move left 90~180deg
          headDeg = map(data.xyJoy[0], 0, joyOffset[0], 180, 90);
        } else {
          headDeg = 90;
        }
        svHead.write(headDeg);

        // wallking
        if (data.xyJoy[1] > joyOffset[1] + joyThreshold) {  // walk forward
          speed = map(data.xyJoy[1], joyOffset[1], 0xFFF, 0, 220); // limitted speed (max speed 255) 
          analogWrite(leg1Pin, speed);
          analogWrite(leg2Pin, 0);
        } else if (data.xyJoy[1] < joyOffset[1] - joyThreshold) {  // walk backward
          speed = map(data.xyJoy[1], 0, joyOffset[1], 220, 0);
          analogWrite(leg1Pin, 0);
          analogWrite(leg2Pin, speed);
        } else {
          analogWrite(leg1Pin, 0);
          analogWrite(leg2Pin, 0);
        }
      }

      // moving right arm ( A: up, B: down)
      switch (data.abcdBtn >> 2 * 3 & B00000011) {  // A button
        case 1:                                     // up
          armRup = true;
          break;
        case 2:  // stop
          armRup = false;
          break;
        default:
          break;
      }

      switch (data.abcdBtn >> 2 * 2 & B00000011) {  // B button
        case 1:                                     // down
          armRdown = true;
          break;
        case 2:  // stop
          armRdown = false;
          break;
        default:
          break;
      }

      // moving left arm
      switch (data.abcdBtn >> 2 * 1 & B00000011) {  // C button
        case 1:                                     // down
          armLdown = true;
          break;
        case 2:  // stop
          armLdown = false;
          break;
        default:
          break;
      }

      switch (data.abcdBtn >> 2 * 0 & B00000011) {  // D button
        case 1:                                     // up
          armLup = true;
          break;
        case 2:  // stop
          armLup = false;
          break;
        default:
          break;
      }
      // chatting (LED + buzzer)
      switch (data.efkBtn >> 2 * 1 & B00000011) {
        case 1:
          chatting = true;
          kBtn = true;
          break;
        case 2:
          kBtn = false;
          break;
        default:
          break;
      }

      // secret button: auto mode1 with singing
      if (!automode1 && armRup && armRdown && armLup && armLdown && !kBtn) {
        doInitVar();
        automode1 = true;
        autoStartTime = millis();
      } else if (millis() - autoStartTime > 3000 && automode1 && (armRup || armRdown || armLup || armLdown || kBtn)) {  // detect stopping 3 sec after starting the automode
        doInitVar();
        automode1 = false;
      }

      if (!automode2 && armRup && armRdown && armLup && armLdown && kBtn) {
        doInitVar();
        automode2 = true;
        autoStartTime = millis();
      } else if (millis() - autoStartTime > 3000 && automode2 && (armRup || armRdown || armLup || armLdown || kBtn)) { //  detect stopping 3 sec after starting the automode
        doInitVar();
        automode2 = false;
      }
    }
  }
};

/******************** < Mandatory functions > ********************/
void setup() {
  doInitialize();
  // Create the BLE Device
  BLEDevice::init(SERVER_NAME);
  // Create the BLE Server
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new ServerCallbacks());
  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);
  // Create a BLE Characteristic
  pCharacteristicRX = pService->createCharacteristic(
                      CHARACTERISTIC_UUID_RX,
                    BLECharacteristic::PROPERTY_WRITE
                    );
  pCharacteristicRX->setCallbacks(new ReceiveCallbacks());

  // Start the service
  pService->start();
  // Start advertising
  BLEAdvertising *pAdvertising = pServer->getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->start();
  Serial.println("Waiting a central to connect...");
}

void loop() {
  // disconnecting -> reconnecting
  if (!deviceConnected && oldDeviceConnected) {
    Serial.println("BLE disconnected!!");
    
    doInitVar();

    // BLE reconnecting
    delay(500);                     // give the bluetooth stack the chance to get things ready
    BLEDevice::startAdvertising();  // restart advertising
    Serial.println("start advertising");
    oldDeviceConnected = deviceConnected;
  }
  // connecting
  if (deviceConnected && !oldDeviceConnected) {
    oldDeviceConnected = deviceConnected;

    // Sign of successful BLE connection
    digitalWrite(ledPin[1], HIGH);
    delay(200);
    digitalWrite(ledPin[0], LOW);
    digitalWrite(ledPin[1], LOW);
    delay(400);
    digitalWrite(ledPin[0], HIGH);
    digitalWrite(ledPin[1], HIGH);
    delay(200);
    digitalWrite(ledPin[0], LOW);
    digitalWrite(ledPin[1], LOW);
  }

  if (deviceConnected) {
    // pushing A or B
    if (armRup && !armRdown) {
      if (armRDeg < armRMaxDeg && oddCheck % 3 == 0) {
        armRDeg++;
      }
      svArmR.write(armRDeg);
    } else if (!armRup && armRdown && oddCheck % 3 == 0) {
      if (armRDeg > armRMinDeg) {
        armRDeg--;
      }
      svArmR.write(armRDeg);
    }

    // pushing C or D
    if (armLup && !armLdown && oddCheck % 3 == 0) {
      if (armLDeg > armLMinDeg) {
        armLDeg--;
      }
      svArmL.write(armLDeg);
    } else if (!armLup && armLdown && oddCheck % 3 == 0) {
      if (armLDeg < armLMaxDeg) {
        armLDeg++;
      }
      svArmL.write(armLDeg);
    }

    if ((armRup || armRdown || armLup || armLdown) && !automode1 && !automode2) {
      oddCheck++;
    }
    if (!armRup && !armRdown && !armLup && !armLdown && !automode1 && !automode2) {
      oddCheck = 0;
    }

    // chatting
    if (chatting) {
      if (doChatting(true)) {
        chatting = false;
      }
    }

    // helloing
    if (helloing) {
      if (doHello()) {
        helloing = false;
      }
    }

    // singing
    if (singing) {
      if (doSinging()) {
        singing = false;
      }
    }

    // automode1 with singing
    if (automode1) {
      if (!autoState) {
        autoState = true;
        helloing = true;
        singing = true;
        oddCheck = 0;
        lastAutoTime1 = millis();
        lastAutoTime2 = millis();
      }

      if (millis() - lastAutoTime1 > interval1) {
        helloing = true;
        lastAutoTime1 = millis();
      }
      if (millis() - lastAutoTime2 > interval2) {
        singing = true;
        lastAutoTime2 = millis();
      }

    }

    // automode2 without singing
    if (automode2) {
      if (!autoState) {
        autoState = true;
        helloing = true;
        oddCheck = 0;
        lastAutoTime1 = millis();
      }

      if (millis() - lastAutoTime1 > interval1) {
        helloing = true;
        lastAutoTime1 = millis();
      }
    }
  }

  delay(10);
}

/******************** < Other functions > ********************/
/*  Initialization process  */
void doInitialize() {
  Serial.begin(SPI_SPEED);

  pinMode(ledPin[0], OUTPUT);
  pinMode(ledPin[1], OUTPUT);

  digitalWrite(ledPin[0], HIGH);  // Sign of power on

  //ledcSetup(bzCh, 12800, 8);

  svHead.attach(headPin, 600, 2400);
  svHead.write(90);
  svArmR.attach(armRPin, 500, 2400);
  svArmR.write(armRMinDeg);
  svArmL.attach(armLPin, 500, 2400);
  svArmL.write(armLMaxDeg);

  analogWrite(leg1Pin, 0);
  analogWrite(leg2Pin, 0);
}


/* Variable initialization process */
void doInitVar() {
  svHead.write(90);
  svArmR.write(armRMinDeg);
  svArmL.write(armLMaxDeg);

  armRDeg = armRMinDeg;
  armLDeg = armLMaxDeg;

  analogWrite(leg1Pin, 0);
  analogWrite(leg2Pin, 0);

  digitalWrite(ledPin[0], LOW);
  digitalWrite(ledPin[1], LOW);

  ledcDetachPin(bzPin);

  for (int i = 0; i < 2; i++) {
    lastLedState[i] = 0;
    ledStep[i] = 0;
    ledStepEnd[i] = false;
  }

  bzState = false;
  lastBzState = false;
  bzStep = 0;
  bzStepEnd = false;

  chatting = false;
  singing = false;
  helloing = false;
  automode1 = false;
  automode2 = false;
  autoState = false;
  oddCheck = 0;
}

/* blink LED process */
bool blinkLed(int pinNo, unsigned long onTime, unsigned long offTime) {  // On LED during onTime and then off LED during offTime
  int state = digitalRead(ledPin[pinNo]);
  if (state == lastLedState[pinNo]) {
    digitalWrite(ledPin[pinNo], HIGH);
    lastLedTime[pinNo] = millis();
    state = 1;
  }
  if (state == 1 && millis() - lastLedTime[pinNo] > onTime) {
    digitalWrite(ledPin[pinNo], LOW);
    lastLedState[pinNo] = state;
    state = 0;
  }
  if (lastLedState[pinNo] == 1 && millis() - lastLedTime[pinNo] > onTime + offTime) {
    lastLedState[pinNo] = state;
    return true;
  }
  return false;
}

/* output buzzer process */
bool outBuzzer(int pin, int ch, int note, unsigned long onTime, unsigned long offTime) {  // Beep during onTime and then stop during offTime, silence at note=0

  if (!bzState && !lastBzState) {
    ledcAttachPin(pin, ch);
    if (note != 0) {
      ledcWriteTone(ch, note);
    }
    lastBzTime = millis();
    bzState = true;
  }
  if (bzState && millis() - lastBzTime > onTime) {
    ledcDetachPin(pin);
    lastBzState = bzState;
    bzState = false;
  }
  if (lastBzState & millis() - lastBzTime > onTime + offTime) {
    lastBzState = bzState;
    return true;
  }
  return false;
}

/* robot chatting process */
bool doChatting(bool bzFlag) {                                           // beep at bzFlag = true
  unsigned long onTime0[] = { 100, 100, 200, 100, 20, 20, 10, 40, 40 };  // for LED1
  unsigned long offTime0[] = { 100, 200, 100, 400, 20, 15, 15, 20, 0 };
  unsigned long onTime1[] = { 0, 200, 100, 200, 40, 65, 20, 20 };  // for LED2
  unsigned long offTime1[] = { 100, 100, 100, 500, 20, 20, 20, 0 };
  unsigned long onBzTime[] = { 200, 100, 200 };  // for buzzer
  unsigned long offBzTime[] = { 20, 980, 0 };
  int bzNote[] = { NOTE_CS7, NOTE_CS7, NOTE_B7 };

  // upper LED
  if (!ledStepEnd[0] && ledStep[0] < sizeof(onTime0) / sizeof(onTime0[0])) {
    if (blinkLed(0, onTime0[ledStep[0]], offTime0[ledStep[0]])) {
      ledStep[0]++;
    }
    ledStepEnd[0] = false;
  } else if (!ledStepEnd[0] && ledStep[0] >= sizeof(onTime0) / sizeof(onTime0[0])) {
    ledStep[0] = 0;
    ledStepEnd[0] = true;
  }

  // lower LED
  if (!ledStepEnd[1] && ledStep[1] < sizeof(onTime1) / sizeof(onTime1[1])) {
    if (blinkLed(1, onTime1[ledStep[1]], offTime1[ledStep[1]])) {
      ledStep[1]++;
    }
    ledStepEnd[1] = false;
  } else if (!ledStepEnd[1] && ledStep[1] >= sizeof(onTime1) / sizeof(onTime1[0])) {
    ledStep[1] = 0;
    ledStepEnd[1] = true;
  }

  // Buzzer
  if (bzFlag) {
    if (!bzStepEnd && bzStep < sizeof(onBzTime) / sizeof(onBzTime[0])) {
      if (outBuzzer(bzPin, bzCh, bzNote[bzStep], onBzTime[bzStep], offBzTime[bzStep])) {
        bzStep++;
      }
      bzStepEnd = false;
    } else if (!bzStepEnd && bzStep >= sizeof(onBzTime) / sizeof(onBzTime[0])) {
      bzStep = 0;
      bzStepEnd = true;
    }
  }

  // check if all steps have been completed
  if (bzFlag) {
    if (ledStepEnd[0] && ledStepEnd[1] && bzStepEnd) {
      ledStepEnd[0] = false;
      ledStepEnd[1] = false;
      bzStepEnd = false;
      return true;
    } else {
      return false;
    }
  } else {
    if (ledStepEnd[0] && ledStepEnd[1]) {
      ledStepEnd[0] = false;
      ledStepEnd[1] = false;
      return true;
    } else {
      return false;
    }
  }
}

/* singing "Carrying you" process */
bool doSinging() {
  float speed = 1900;
  int bzNote[] = {
    NOTE_A4, NOTE_B4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_E5, NOTE_B4,
    NOTE_E4, NOTE_A4, NOTE_G4, NOTE_A4, NOTE_C5, NOTE_G4,
    NOTE_E4, NOTE_F4, NOTE_E4, NOTE_F4, NOTE_C5, NOTE_E4,
    0, NOTE_C5, NOTE_C5, NOTE_C5, NOTE_B4, NOTE_FS4, NOTE_FS4, NOTE_B4, NOTE_B4,

    NOTE_A4, NOTE_B4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_E5, NOTE_B4,
    NOTE_E4, NOTE_A4, NOTE_G4, NOTE_A4, NOTE_C5, NOTE_G4,
    NOTE_E4, NOTE_F4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_D5, NOTE_E5, NOTE_C5,
    NOTE_C5, NOTE_B4, NOTE_A4, NOTE_B4, NOTE_GS4, NOTE_A4,

    NOTE_C5, NOTE_D5, NOTE_E5, NOTE_D5, NOTE_E5, NOTE_G5, NOTE_D5,
    NOTE_G4, NOTE_G4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_E5, NOTE_E5,
    NOTE_A4, NOTE_B4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_D5, NOTE_C5, NOTE_G4, NOTE_G4,
    NOTE_F5, NOTE_E5, NOTE_D5, NOTE_C5, NOTE_E5, 0,

    NOTE_E5, NOTE_A5, NOTE_G5, NOTE_E5, NOTE_D5, NOTE_C5, 0,
    NOTE_C5, NOTE_D5, NOTE_C5, NOTE_D5, NOTE_G5, NOTE_E5,
    NOTE_E5, NOTE_A5, NOTE_G5, NOTE_E5, NOTE_D5, NOTE_C5, 0,
    NOTE_C5, NOTE_D5, NOTE_C5, NOTE_D5, NOTE_B4, NOTE_A4,

    NOTE_A4, NOTE_B4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_E5, NOTE_B4,
    NOTE_E4, NOTE_A4, NOTE_G4, NOTE_A4, NOTE_C5, NOTE_G4,
    NOTE_E4, NOTE_F4, NOTE_E4, NOTE_F4, NOTE_C5, NOTE_E4,
    0, NOTE_C5, NOTE_C5, NOTE_C5, NOTE_B4, NOTE_FS4, NOTE_FS4, NOTE_B4, NOTE_B4,

    NOTE_A4, NOTE_B4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_E5, NOTE_B4,
    NOTE_E4, NOTE_A4, NOTE_G4, NOTE_A4, NOTE_C5, NOTE_G4,
    NOTE_E4, NOTE_F4, NOTE_C5, NOTE_B4, NOTE_C5, NOTE_D5, NOTE_E5, NOTE_C5,
    NOTE_C5, NOTE_B4, NOTE_A4, NOTE_B4, NOTE_GS4, NOTE_A4
  };

  float noteDurations[] = {
    8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4.0 / 1.5, 8, 8, 4.0 / 1.5, 2,
    8, 8, 8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,

    8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4, 8, 4.0 / 1.5, 4, 4, 8, 4.0 / 2.5,
    8, 8, 4, 4, 4, 4.0 / 3.0,

    8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    8, 8, 4.0 / 1.5, 8, 4, 4, 1,
    8, 8, 4, 8, 8, 4, 4.0 / 1.5, 8, 2,
    4, 4, 4, 4, 1.0 / 1.5, 4,

    4, 2, 2, 8, 8, 2, 8,
    8, 4, 8, 4.0 / 1.5, 4, 4.0 / 3.0,
    4, 2, 2, 8, 8, 2, 8,
    8, 4, 8, 4.0 / 1.5, 4, 4.0 / 3.0,

    8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 2.5,
    4, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4.0 / 1.5, 8, 8, 4.0 / 1.5, 2,
    8, 8, 8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,

    8, 8, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4.0 / 1.5, 8, 4, 4, 4.0 / 3.0,
    4, 4, 8, 4.0 / 1.5, 4, 4, 8, 4.0 / 2.5,
    8, 8, 4, 4, 4, 1
  };

  if (!bzStepEnd && bzStep < sizeof(bzNote) / sizeof(bzNote[0])) {
    if (outBuzzer(bzPin, bzCh, bzNote[bzStep], round(speed / noteDurations[bzStep] * 0.85), round(speed / noteDurations[bzStep] * 0.15))) {
      bzStep++;
    }
    bzStepEnd = false;
  } else if (!bzStepEnd && bzStep >= sizeof(bzNote) / sizeof(bzNote[0])) {
    bzStep = 0;
    bzStepEnd = true;
  }

  if (bzStepEnd) {
    bzStepEnd = false;
    return true;
  } else {
    return false;
  }
}

/* moving automatically process */
bool doHello() {
  Serial.print(doHelloStep);
  Serial.print(",");
  Serial.println(oddCheck);
  switch (doHelloStep) {
    case 0:  //turn right      
      oddCheck++;
      if (headDeg >= 40 && oddCheck % 2 == 0) {
        headDeg--;
        svHead.write(headDeg);
      }
      if (headDeg < 40) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 1:  // raise the right arm
      oddCheck++;
      if (armRDeg <= armRMaxDeg && oddCheck % 2 == 0) {
        armRDeg++;
        svArmR.write(armRDeg);
      }
      if (armRDeg > armRMaxDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 2:  // waiting
      if (millis() - lastHelloTime > 2000) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 3:  // lower the right arm
      oddCheck++;
      if (armRDeg >= armRMinDeg && oddCheck % 2 == 0) {
        armRDeg--;
        svArmR.write(armRDeg);
      }
      if (armRDeg < armRMinDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 4:  // face forward
      oddCheck++;
      if (headDeg <= 90 && oddCheck % 2 == 0) {
        headDeg++;
        svHead.write(headDeg);
      }
      if (headDeg > 90) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 5:  // waiting
      if (millis() - lastHelloTime > 2000) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 6:  // trun left
      oddCheck++;
      if (headDeg <= 140 && oddCheck % 2 == 0) {
        headDeg++;
        svHead.write(headDeg);
      }
      if (headDeg > 140) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 7:  // raise the right arm
      oddCheck++;
      if (armRDeg <= armRMaxDeg && oddCheck % 2 == 0) {
        armRDeg++;
        svArmR.write(armRDeg);
      }
      if (armRDeg > armRMaxDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 8:  // waiting
      if (millis() - lastHelloTime > 2000) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 9:  // lower the right arm
      oddCheck++;
      if (armRDeg >= armRMinDeg && oddCheck % 2 == 0) {
        armRDeg--;
        svArmR.write(armRDeg);
      }
      if (armRDeg < armRMinDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 10:  // face forward
      oddCheck++;
      if (headDeg >= 90 && oddCheck % 2 == 0) {
        headDeg--;
        svHead.write(headDeg);
      }
      if (headDeg < 90) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 11:  // waiting
      if (millis() - lastHelloTime > 2000) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 12:  // raise the arms
      oddCheck++;
      if (armRDeg <= armRMaxDeg && oddCheck % 2 == 0) {
        armRDeg++;
        svArmR.write(armRDeg);
      }
      if (armLDeg >= armLMinDeg && oddCheck % 2 == 0) {
        armLDeg--;
        svArmL.write(armLDeg);
      }
      if (armRDeg > armRMaxDeg && armLDeg < armLMinDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 13:  // waiting
      if (millis() - lastHelloTime > 500) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 14:  // lower the arms halfway
      oddCheck++;
      if (armRDeg >= 90 && oddCheck % 2 == 0) {
        armRDeg--;
        svArmR.write(armRDeg);
      }
      if (armLDeg <= 70 && oddCheck % 2 == 0) {
        armLDeg++;
        svArmL.write(armLDeg);
      }
      if (armRDeg < 90 && armLDeg > 70) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 15:  // waiting
      if (millis() - lastHelloTime > 500) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 16:  // raise the arms
      oddCheck++;
      if (armRDeg <= armRMaxDeg && oddCheck % 2 == 0) {
        armRDeg++;
        svArmR.write(armRDeg);
      }
      if (armLDeg >= armLMinDeg && oddCheck % 2 == 0) {
        armLDeg--;
        svArmL.write(armLDeg);
      }
      if (armRDeg > armRMaxDeg && armLDeg < armLMinDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 17:  // waiting
      if (millis() - lastHelloTime > 2000) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 18:  // chatting
      if (singing) {
        if (doChatting(false)) {
          oddCheck = 0;
          lastHelloTime = millis();
          doHelloStep++;
        }
      } else{
        if (doChatting(true)) {
          oddCheck = 0;
          lastHelloTime = millis();
          doHelloStep++;
        }
      }
      break;
    case 19:  // lower the arms
      oddCheck++;
      if (armRDeg >= armRMinDeg && oddCheck % 2 == 0) {
        armRDeg--;
        svArmR.write(armRDeg);
      }
      if (armLDeg <= armLMaxDeg && oddCheck % 2 == 0) {
        armLDeg++;
        svArmL.write(armLDeg);
      }
      if (armRDeg < armRMinDeg && armLDeg > armLMaxDeg) {
        oddCheck = 0;
        lastHelloTime = millis();
        doHelloStep++;
      }
      break;
    case 20:
      oddCheck = 0;
      lastHelloTime = millis();
      doHelloStep = 0;
      helloing = false;
      return true;
      break;
    default:
      break;
  }

  return false;
}
